@@ -1,5 +1,5 @@
  /* BoCA - BonkEnc Component Architecture
-  * Copyright (C) 2007-2020 Robert Kausch <robert.kausch@freac.org>
+  * Copyright (C) 2007-2021 Robert Kausch <robert.kausch@freac.org>
   *
   * This program is free software; you can redistribute it and/or
   * modify it under the terms of the GNU General Public License as
@@ -133,8 +133,8 @@ Error BoCA::DecoderCueSheet::GetStreamInfo(const String &streamURI, Track &track
 		{
 			String	 genre;
 
-			if (line.Contains("\"")) genre = line.SubString(line.Find("\"") + 1, line.FindLast("\"") - line.Find("\"") - 1);
-			else			 genre = line.Tail(line.Length() - 10);
+			if (line.Contains("\"")) genre = UnescapeString(line.SubString(line.Find("\"") + 1, line.FindLast("\"") - line.Find("\"") - 1));
+			else			 genre = UnescapeString(line.Tail(line.Length() - 10));
 
 			if (!readInfoTags || preferCueSheets) info.genre = genre;
 
@@ -147,8 +147,8 @@ Error BoCA::DecoderCueSheet::GetStreamInfo(const String &streamURI, Track &track
 		{
 			String	 comment;
 
-			if (line.Contains("\"")) comment = line.SubString(line.Find("\"") + 1, line.FindLast("\"") - line.Find("\"") - 1);
-			else			 comment = line.Tail(line.Length() - 12);
+			if (line.Contains("\"")) comment = UnescapeString(line.SubString(line.Find("\"") + 1, line.FindLast("\"") - line.Find("\"") - 1));
+			else			 comment = UnescapeString(line.Tail(line.Length() - 12));
 
 			if (!readInfoTags || preferCueSheets) info.comment = comment;
 
@@ -172,8 +172,8 @@ Error BoCA::DecoderCueSheet::GetStreamInfo(const String &streamURI, Track &track
 		{
 			String	 artist;
 
-			if (line.Contains("\"")) artist = line.SubString(line.Find("\"") + 1, line.FindLast("\"") - line.Find("\"") - 1);
-			else			 artist = line.Tail(line.Length() - 10);
+			if (line.Contains("\"")) artist = UnescapeString(line.SubString(line.Find("\"") + 1, line.FindLast("\"") - line.Find("\"") - 1));
+			else			 artist = UnescapeString(line.Tail(line.Length() - 10));
 
 			if (!readInfoTags || preferCueSheets)
 			{
@@ -190,8 +190,8 @@ Error BoCA::DecoderCueSheet::GetStreamInfo(const String &streamURI, Track &track
 		{
 			String	 songwriter;
 
-			if (line.Contains("\"")) songwriter = line.SubString(line.Find("\"") + 1, line.FindLast("\"") - line.Find("\"") - 1);
-			else			 songwriter = line.Tail(line.Length() - 11);
+			if (line.Contains("\"")) songwriter = UnescapeString(line.SubString(line.Find("\"") + 1, line.FindLast("\"") - line.Find("\"") - 1));
+			else			 songwriter = UnescapeString(line.Tail(line.Length() - 11));
 
 			if (!readInfoTags || preferCueSheets) info.SetOtherInfo(INFO_COMPOSER, songwriter);
 
@@ -202,8 +202,8 @@ Error BoCA::DecoderCueSheet::GetStreamInfo(const String &streamURI, Track &track
 		{
 			String	 title;
 
-			if (line.Contains("\"")) title = line.SubString(line.Find("\"") + 1, line.FindLast("\"") - line.Find("\"") - 1);
-			else			 title = line.Tail(line.Length() - 6);
+			if (line.Contains("\"")) title = UnescapeString(line.SubString(line.Find("\"") + 1, line.FindLast("\"") - line.Find("\"") - 1));
+			else			 title = UnescapeString(line.Tail(line.Length() - 6));
 
 			if (!readInfoTags || preferCueSheets)
 			{
@@ -271,7 +271,7 @@ Error BoCA::DecoderCueSheet::GetStreamInfo(const String &streamURI, Track &track
 										       msf.GetNth(1).ToInt()	  * format.rate +
 										       msf.GetNth(2).ToInt()	  * format.rate / 75, fileLength);
 
-							if (samplePos >= iTrack.sampleOffset)
+							if (samplePos > iTrack.sampleOffset)
 							{
 								iTrack.length	= samplePos - iTrack.sampleOffset;
 								iTrack.fileSize	= Math::Round(Float(iTrack.fileSize) / fileLength * iTrack.length);
@@ -287,7 +287,7 @@ Error BoCA::DecoderCueSheet::GetStreamInfo(const String &streamURI, Track &track
 
 			/* Add previous track.
 			 */
-			if (iTrack.length != -1 || iTrack.approxLength != -1)
+			if (iTrack.length > 0 || iTrack.approxLength > 0)
 			{
 				iTrack.SetFormat(format);
 				iTrack.SetInfo(info);
@@ -602,6 +602,11 @@ Bool BoCA::DecoderCueSheet::AddTrack(const Track &track, Array<Track> &tracks) c
 	return True;
 }
 
+String BoCA::DecoderCueSheet::UnescapeString(const String &string)
+{
+	return string.Replace(" // ", "\n").Replace("''", "\"");
+}
+
 BoCA::DecoderCueSheet::DecoderCueSheet()
 {
 	configLayer = NIL;
