@@ -1,5 +1,5 @@
  /* BoCA - BonkEnc Component Architecture
-  * Copyright (C) 2007-2020 Robert Kausch <robert.kausch@freac.org>
+  * Copyright (C) 2007-2021 Robert Kausch <robert.kausch@freac.org>
   *
   * This program is free software; you can redistribute it and/or
   * modify it under the terms of the GNU General Public License as
@@ -31,7 +31,7 @@ const String &BoCA::DecoderFDKAAC::GetComponentSpecs()
 											\
 		  <?xml version=\"1.0\" encoding=\"UTF-8\"?>				\
 		  <component>								\
-		    <name>Fraunhofer FDK AAC Decoder</name>				\
+		    <name>FDK-AAC Decoder %VERSION%</name>				\
 		    <version>1.0</version>						\
 		    <id>fdkaac-dec</id>							\
 		    <type>decoder</type>						\
@@ -67,6 +67,12 @@ const String &BoCA::DecoderFDKAAC::GetComponentSpecs()
 		  </component>								\
 											\
 		");
+
+		UnsignedInt32	 version = GetDecoderVersion();
+
+		componentSpecs.Replace("%VERSION%", String("v").Append(String::FromInt((version >> 24) & 0xff)).Append(".")
+							       .Append(String::FromInt((version >> 16) & 0xff)).Append(".")
+							       .Append(String::FromInt((version >>  8) & 0xff)));
 	}
 
 	return componentSpecs;
@@ -91,7 +97,7 @@ Bool BoCA::DecoderFDKAAC::CanOpenStream(const String &streamURI)
 
 	if (mp4v2dll != NIL && (in.InputNumberRaw(8) & 0xFFFFFFFF) == 'ftyp')
 	{
-		MP4FileHandle	 mp4File  = ex_MP4Read(streamURI.ConvertTo("UTF-8"));
+		MP4FileHandle	 mp4File  = ex_MP4ReadProvider(streamURI.ConvertTo("UTF-8"), NIL);
 		Int		 mp4Track = GetAudioTrack(mp4File);
 
 		if (mp4Track >= 0 && ex_MP4GetSampleSize(mp4File, mp4Track, 1) > 0)
@@ -140,7 +146,7 @@ Error BoCA::DecoderFDKAAC::GetStreamInfo(const String &streamURI, Track &track)
 		track.fileSize	= File(streamURI).GetFileSize();
 		track.length	= -1;
 
-		MP4FileHandle	 mp4File  = ex_MP4Read(streamURI.ConvertTo("UTF-8"));
+		MP4FileHandle	 mp4File  = ex_MP4ReadProvider(streamURI.ConvertTo("UTF-8"), NIL);
 		Int		 mp4Track = GetAudioTrack(mp4File);
 
 		if (mp4Track >= 0 && ex_MP4GetSampleSize(mp4File, mp4Track, 1) > 0)
@@ -382,13 +388,30 @@ BoCA::DecoderFDKAAC::~DecoderFDKAAC()
 {
 }
 
+UnsignedInt32 BoCA::DecoderFDKAAC::GetDecoderVersion()
+{
+	LIB_INFO	 info[FDK_MODULE_LAST];
+
+	FDKinitLibInfo(info);
+	ex_aacDecoder_GetLibInfo(info);
+
+	for (Int i = 0; i < FDK_MODULE_LAST; i++)
+	{
+		if (info[i].module_id != FDK_AACDEC) continue;
+
+		return info[i].version;
+	}
+
+	return 0;
+}
+
 Bool BoCA::DecoderFDKAAC::Activate()
 {
 	InStream	 in(STREAM_DRIVER, driver);
 
 	if ((in.InputNumberRaw(8) & 0xFFFFFFFF) == 'ftyp')
 	{
-		mp4File	 = ex_MP4Read(track.fileName.ConvertTo("UTF-8"));
+		mp4File	 = ex_MP4ReadProvider(track.fileName.ConvertTo("UTF-8"), NIL);
 		mp4Track = GetAudioTrack(mp4File);
 
 		if (mp4Track == -1) return False;
