@@ -1,5 +1,5 @@
  /* BoCA - BonkEnc Component Architecture
-  * Copyright (C) 2007-2020 Robert Kausch <robert.kausch@freac.org>
+  * Copyright (C) 2007-2021 Robert Kausch <robert.kausch@freac.org>
   *
   * This program is free software; you can redistribute it and/or
   * modify it under the terms of the GNU General Public License as
@@ -38,7 +38,7 @@ const String &BoCA::EncoderOpus::GetComponentSpecs()
 		      <extension>oga</extension>							\
 		      <tag id=\"vorbis-tag\" mode=\"other\">Vorbis Comment</tag>			\
 		    </format>										\
-		    <input bits=\"16\" channels=\"1-8\" rate=\"8000,12000,16000,24000,48000\"/>		\
+		    <input bits=\"16\" channels=\"1-8\"/>						\
 		    <parameters>									\
 		      <range name=\"Bitrate\" argument=\"--bitrate %VALUE\" default=\"128\">		\
 			<min>6</min>									\
@@ -108,6 +108,8 @@ BoCA::EncoderOpus::EncoderOpus()
 	configLayer	= NIL;
 	config		= NIL;
 
+	converter	= NIL;
+
 	frameSize	= 0;
 	preSkip		= 0;
 
@@ -143,6 +145,30 @@ Bool BoCA::EncoderOpus::Activate()
 
 	ConvertArguments(config);
 
+	/* Get target format.
+	 */
+	targetFormat = format;
+
+	if	(format.rate <=  8000) targetFormat.rate =  8000;
+	else if (format.rate <= 12000) targetFormat.rate = 12000;
+	else if (format.rate <= 16000) targetFormat.rate = 16000;
+	else if (format.rate <= 24000) targetFormat.rate = 24000;
+	else			       targetFormat.rate = 48000;
+
+	/* Create and init format converter component.
+	 */
+	converter = new FormatConverter(format, targetFormat);
+
+	if (converter->GetErrorState() == True)
+	{
+		errorState  = True;
+		errorString = converter->GetErrorString();
+
+		delete converter;
+
+		return False;
+	}
+
 	/* Init Ogg stream.
 	 */
 	Math::RandomSeed();
@@ -153,7 +179,7 @@ Bool BoCA::EncoderOpus::Activate()
 	 */
 	OpusHeader	 setup;
 
-	strncpy(setup.codec_id, "OpusHead", 8);
+	memcpy(setup.codec_id, "OpusHead", 8);
 
 	setup.version_id  = 1;
 	setup.nb_channels = format.channels;
@@ -169,7 +195,7 @@ Bool BoCA::EncoderOpus::Activate()
 	int	 streams = 0;
 	int	 coupled = 0;
 
-	OpusMSEncoder	*encoder = ex_opus_multistream_surround_encoder_create(format.rate, setup.nb_channels, setup.channel_mapping, &streams, &coupled, setup.stream_map, OPUS_APPLICATION_AUDIO, &error);
+	OpusMSEncoder	*encoder = ex_opus_multistream_surround_encoder_create(targetFormat.rate, setup.nb_channels, setup.channel_mapping, &streams, &coupled, setup.stream_map, OPUS_APPLICATION_AUDIO, &error);
 
 	setup.nb_streams = streams;
 	setup.nb_coupled = coupled;
@@ -178,9 +204,9 @@ Bool BoCA::EncoderOpus::Activate()
 	 */
 	ex_opus_multistream_encoder_ctl(encoder, OPUS_GET_LOOKAHEAD(&preSkip));
 
-	setup.preskip = preSkip * (48000 / format.rate);
+	setup.preskip = preSkip * (48000 / targetFormat.rate);
 
-	frameSize     = Math::Round(Float(format.rate) / (1000000.0 / config->GetIntValue(ConfigureOpus::ConfigID, "FrameSize", 20000)));
+	frameSize     = Math::Round(Float(targetFormat.rate) / (1000000.0 / config->GetIntValue(ConfigureOpus::ConfigID, "FrameSize", 20000)));
 	totalSamples  = 0;
 
 	ex_opus_multistream_encoder_destroy(encoder);
@@ -200,7 +226,7 @@ Bool BoCA::EncoderOpus::Activate()
 
 	ex_ogg_stream_packetin(&os, &header);
 
-	/* Write Vorbis comment header
+	/* Write Vorbis Comment header
 	 */
 	{
 		Buffer<unsigned char>	 vcBuffer;
@@ -212,10 +238,10 @@ Bool BoCA::EncoderOpus::Activate()
 		info.album_gain = NIL;
 		info.album_peak = NIL;
 
-		/* Render actual Vorbis comment tag.
+		/* Render actual Vorbis Comment tag.
 		 *
 		 * An empty tag containing only the vendor string
-		 * is rendered if Vorbis comments are disabled.
+		 * is rendered if Vorbis Comment tags are disabled.
 		 */
 		AS::Registry		&boca = AS::Registry::Get();
 		AS::TaggerComponent	*tagger = (AS::TaggerComponent *) boca.CreateComponentByID("vorbis-tag");
@@ -260,7 +286,7 @@ Bool BoCA::EncoderOpus::Activate()
 
 	/* Start up worker threads.
 	 */
-	for (Int i = 0; i < numberOfThreads; i++) workers.Add(new SuperWorker(config, format));
+	for (Int i = 0; i < numberOfThreads; i++) workers.Add(new SuperWorker(config, targetFormat));
 
 	foreach (SuperWorker *worker, workers) worker->Start();
 
@@ -269,6 +295,22 @@ Bool BoCA::EncoderOpus::Activate()
 
 Bool BoCA::EncoderOpus::Deactivate()
 {
+	/* Flush and clean up format converter.
+	 */
+	Buffer<UnsignedByte>	 buffer;
+
+	converter->Finish(buffer);
+
+	delete converter;
+
+	/* Append remaining samples to samples buffer.
+	 */
+	Int	 samples = buffer.Size() / 2;
+
+	samplesBuffer.Resize(samplesBuffer.Size() + samples);
+
+	memcpy(samplesBuffer + samplesBuffer.Size() - samples, buffer, buffer.Size());
+
 	/* Output remaining samples to encoder.
 	 */
 	EncodeFrames(True);
@@ -289,7 +331,20 @@ Bool BoCA::EncoderOpus::Deactivate()
 
 	/* Fix chapter marks in Vorbis Comments.
 	 */
-	FixChapterMarks();
+	if (config->GetIntValue("Tags", "EnableVorbisComment", True) && track.tracks.Length() > 0 && config->GetIntValue("Tags", "WriteChapters", True))
+	{
+		driver->Close();
+
+		AS::Registry		&boca = AS::Registry::Get();
+		AS::TaggerComponent	*tagger = (AS::TaggerComponent *) boca.CreateComponentByID("vorbis-tag");
+
+		if (tagger != NIL)
+		{
+			tagger->UpdateStreamInfo(track.outputFile, track);
+
+			boca.DeleteComponent(tagger);
+		}
+	}
 
 	return True;
 }
@@ -306,6 +361,10 @@ Int BoCA::EncoderOpus::WriteData(Buffer<UnsignedByte> &data)
 	else if (format.channels == 7) Utilities::ChangeChannelOrder(data, format, Channel::Default_6_1, Channel::Vorbis_6_1);
 	else if (format.channels == 8) Utilities::ChangeChannelOrder(data, format, Channel::Default_7_1, Channel::Vorbis_7_1);
 
+	/* Perform sample rate conversion.
+	 */
+	converter->Transform(data);
+
 	/* Copy data to samples buffer.
 	 */
 	Int	 samples = data.Size() / 2;
@@ -321,8 +380,6 @@ Int BoCA::EncoderOpus::WriteData(Buffer<UnsignedByte> &data)
 
 Int BoCA::EncoderOpus::EncodeFrames(Bool flush)
 {
-	const Format	&format = track.GetFormat();
-
 	/* Pad end of stream with empty samples.
 	 */
 	Int	 nullSamples = 0;
@@ -331,11 +388,11 @@ Int BoCA::EncoderOpus::EncodeFrames(Bool flush)
 	{
 		nullSamples = preSkip;
 
-		if ((samplesBuffer.Size() / format.channels + preSkip) % frameSize > 0) nullSamples += frameSize - (samplesBuffer.Size() / format.channels + preSkip) % frameSize;
+		if ((samplesBuffer.Size() / targetFormat.channels + preSkip) % frameSize > 0) nullSamples += frameSize - (samplesBuffer.Size() / targetFormat.channels + preSkip) % frameSize;
 
-		samplesBuffer.Resize(samplesBuffer.Size() + nullSamples * format.channels);
+		samplesBuffer.Resize(samplesBuffer.Size() + nullSamples * targetFormat.channels);
 
-		memset(((signed short *) samplesBuffer) + samplesBuffer.Size() - nullSamples * format.channels, 0, sizeof(short) * nullSamples * format.channels);
+		memset(((signed short *) samplesBuffer) + samplesBuffer.Size() - nullSamples * targetFormat.channels, 0, sizeof(short) * nullSamples * targetFormat.channels);
 	}
 
 	/* Pass samples to workers.
@@ -344,7 +401,7 @@ Int BoCA::EncoderOpus::EncodeFrames(Bool flush)
 	Int	 framesProcessed = 0;
 	Int	 dataLength	 = 0;
 
-	Int	 samplesPerFrame = frameSize * format.channels;
+	Int	 samplesPerFrame = frameSize * targetFormat.channels;
 
 	if (flush) framesToProcess = Math::Floor(samplesBuffer.Size() / samplesPerFrame);
 
@@ -395,8 +452,7 @@ Int BoCA::EncoderOpus::EncodeFrames(Bool flush)
 
 Int BoCA::EncoderOpus::ProcessPackets(const Buffer<unsigned char> &packets, const Array<Int> &packetSizes, Bool first, Bool flush, Int nullSamples)
 {
-	const Format	&format = track.GetFormat();
-	Int		 offset = 0;
+	Int	 offset = 0;
 
 	if (!first) for (Int i = 0; i < overlap; i++) offset += packetSizes.GetNth(i);
 
@@ -411,7 +467,7 @@ Int BoCA::EncoderOpus::ProcessPackets(const Buffer<unsigned char> &packets, cons
 		op.bytes      = packetSizes.GetNth(i);
 		op.b_o_s      = first && i == 0;
 		op.e_o_s      = flush && i == packetSizes.Length() - 1;
-		op.granulepos = (op.e_o_s ? totalSamples + preSkip - nullSamples : totalSamples) * (48000 / format.rate);
+		op.granulepos = (op.e_o_s ? totalSamples + preSkip - nullSamples : totalSamples) * (48000 / targetFormat.rate);
 		op.packetno   = 0;
 
 		ex_ogg_stream_packetin(&os, &op);	
@@ -443,94 +499,6 @@ Int BoCA::EncoderOpus::WriteOggPackets(Bool flush)
 	return bytes;
 }
 
-Bool BoCA::EncoderOpus::FixChapterMarks()
-{
-	if (track.tracks.Length() == 0 || !config->GetIntValue("Tags", "WriteChapters", True)) return True;
-
-	driver->Seek(0);
-
-	/* Skip first Ogg page and read second into buffer.
-	 */
-	Buffer<UnsignedByte>	 buffer;
-	Int			 position;
-	ogg_page		 og;
-
-	for (Int i = 0; i < 2; i++)
-	{
-		driver->Seek(driver->GetPos() + 26);
-
-		Int		 dataSize    = 0;
-		UnsignedByte	 segments    = 0;
-		UnsignedByte	 segmentSize = 0;
-
-		driver->ReadData(&segments, 1);
-
-		for (Int i = 0; i < segments; i++) { driver->ReadData(&segmentSize, 1); dataSize += segmentSize; }
-
-		buffer.Resize(27 + segments + dataSize);
-		position = driver->GetPos() - segments - 27;
-
-		driver->Seek(position);
-		driver->ReadData(buffer, buffer.Size());
-
-		og.header     = buffer;
-		og.header_len = 27 + segments;
-		og.body	      = buffer + og.header_len;
-		og.body_len   = dataSize;
-	}
-
-	/* Update chapter marks.
-	 */
-	if (buffer.Size() > 0)
-	{
-		Int64	 offset = 0;
-
-		for (Int i = 0; i < track.tracks.Length(); i++)
-		{
-			const Track	&chapterTrack  = track.tracks.GetNth(i);
-			const Format	&chapterFormat = chapterTrack.GetFormat();
-
-			for (Int b = 0; b < buffer.Size() - 23; b++)
-			{
-				if (buffer[b + 0] != 'C' || buffer[b + 1] != 'H' || buffer[b + 2] != 'A' || buffer[b +  3] != 'P' ||
-				    buffer[b + 4] != 'T' || buffer[b + 5] != 'E' || buffer[b + 6] != 'R' || buffer[b + 10] != '=') continue;
-
-				String	 id;
-
-				id[0] = buffer[b + 7];
-				id[1] = buffer[b + 8];
-				id[2] = buffer[b + 9];
-
-				if (id.ToInt() != i + 1) continue;
-
-				String	 value	= String(offset / chapterFormat.rate / 60 / 60 < 10 ? "0" : "").Append(String::FromInt(offset / chapterFormat.rate / 60 / 60)).Append(":")
-						 .Append(offset / chapterFormat.rate / 60 % 60 < 10 ? "0" : "").Append(String::FromInt(offset / chapterFormat.rate / 60 % 60)).Append(":")
-						 .Append(offset / chapterFormat.rate % 60      < 10 ? "0" : "").Append(String::FromInt(offset / chapterFormat.rate % 60)).Append(".")
-						 .Append(Math::Round(offset % chapterFormat.rate * 1000.0 / chapterFormat.rate) < 100 ?
-							(Math::Round(offset % chapterFormat.rate * 1000.0 / chapterFormat.rate) <  10 ?  "00" : "0") : "").Append(String::FromInt(Math::Round(offset % chapterFormat.rate * 1000.0 / chapterFormat.rate)));
-
-				for (Int p = 0; p < 12; p++) buffer[b + 11 + p] = value[p];
-
-				break;
-			}
-
-			if	(chapterTrack.length	   >= 0) offset += chapterTrack.length;
-			else if (chapterTrack.approxLength >= 0) offset += chapterTrack.approxLength;
-		}
-
-		/* Write page back to file.
-		 */
-		ex_ogg_page_checksum_set(&og);
-
-		driver->Seek(position);
-		driver->WriteData(buffer, buffer.Size());
-	}
-
-	driver->Seek(driver->GetSize());
-
-	return True;
-}
-
 String BoCA::EncoderOpus::GetOutputFileExtension() const
 {
 	const Config	*config = GetConfiguration();
@@ -549,11 +517,31 @@ Bool BoCA::EncoderOpus::ConvertArguments(Config *config)
 
 	static const String	 encoderID = "opus-enc";
 
+	/* Set default values.
+	 */
+	if (!config->GetIntValue("Settings", "UserSpecifiedConfig", False))
+	{
+		config->SetIntValue(ConfigureOpus::ConfigID, "Mode", 0);
+		config->SetIntValue(ConfigureOpus::ConfigID, "Bandwidth", 0);
+		config->SetIntValue(ConfigureOpus::ConfigID, "PacketLoss", 0);
+		config->SetIntValue(ConfigureOpus::ConfigID, "EnableDTX", False);
+
+		config->SetIntValue(ConfigureOpus::ConfigID, "EnableVBR", True);
+		config->SetIntValue(ConfigureOpus::ConfigID, "EnableConstrainedVBR", False);
+
+		config->SetIntValue(ConfigureOpus::ConfigID, "Bitrate", 128);
+		config->SetIntValue(ConfigureOpus::ConfigID, "Complexity", 10);
+		config->SetIntValue(ConfigureOpus::ConfigID, "FrameSize", 20000);
+	}
+
 	/* Get command line settings.
 	 */
-	Int	 bitrate    = 128;
-	Int	 complexity = 10;
-	Int	 framesize  = 20;
+	Bool	 hardCBR	= config->GetIntValue(encoderID, "Use hard CBR encoding", !config->GetIntValue(ConfigureOpus::ConfigID, "EnableVBR", True));
+	Bool	 constrainedVBR = config->GetIntValue(encoderID, "Use constrained VBR encoding", config->GetIntValue(ConfigureOpus::ConfigID, "EnableConstrainedVBR", False));
+
+	Int	 bitrate	= config->GetIntValue(ConfigureOpus::ConfigID, "Bitrate", 128);
+	Int	 complexity	= config->GetIntValue(ConfigureOpus::ConfigID, "Complexity", 10);
+	Int	 framesize	= config->GetIntValue(ConfigureOpus::ConfigID, "FrameSize", 20000) / 1000;
 
 	if (config->GetIntValue(encoderID, "Set Bitrate", False))	      bitrate	 = config->GetIntValue(encoderID, "Bitrate", bitrate);
 	if (config->GetIntValue(encoderID, "Set Encoding complexity", False)) complexity = config->GetIntValue(encoderID, "Encoding complexity", complexity);
@@ -561,13 +549,8 @@ Bool BoCA::EncoderOpus::ConvertArguments(Config *config)
 
 	/* Set configuration values.
 	 */
-	config->SetIntValue(ConfigureOpus::ConfigID, "Mode", 0);
-	config->SetIntValue(ConfigureOpus::ConfigID, "Bandwidth", 0);
-	config->SetIntValue(ConfigureOpus::ConfigID, "PacketLoss", 0);
-	config->SetIntValue(ConfigureOpus::ConfigID, "EnableDTX", False);
-
-	config->SetIntValue(ConfigureOpus::ConfigID, "EnableVBR", !config->GetIntValue(encoderID, "Use hard CBR encoding", False));
-	config->SetIntValue(ConfigureOpus::ConfigID, "EnableConstrainedVBR", config->GetIntValue(encoderID, "Use constrained VBR encoding", False));
+	config->SetIntValue(ConfigureOpus::ConfigID, "EnableVBR", !hardCBR);
+	config->SetIntValue(ConfigureOpus::ConfigID, "EnableConstrainedVBR", constrainedVBR);
 
 	config->SetIntValue(ConfigureOpus::ConfigID, "Bitrate", Math::Max(6, Math::Min(510, bitrate)));
 	config->SetIntValue(ConfigureOpus::ConfigID, "Complexity", Math::Max(0, Math::Min(10, complexity)));
