@@ -1,5 +1,5 @@
  /* BoCA - BonkEnc Component Architecture
-  * Copyright (C) 2007-2020 Robert Kausch <robert.kausch@freac.org>
+  * Copyright (C) 2007-2021 Robert Kausch <robert.kausch@freac.org>
   *
   * This program is free software; you can redistribute it and/or
   * modify it under the terms of the GNU General Public License as
@@ -133,6 +133,7 @@ Error BoCA::TaggerAPEv2::RenderBuffer(Buffer<UnsignedByte> &buffer, const Track
 		else if	(key == INFO_CONTENTGROUP)   { RenderAPEItem("Grouping",       value, buffer); numItems++; }
 		else if	(key == INFO_SUBTITLE)	     { RenderAPEItem("Subtitle",       value, buffer); numItems++; }
 
+		else if	(key == INFO_BAND)	     { RenderAPEItem("Ensemble",       value, buffer); numItems++; }
 		else if	(key == INFO_PERFORMER)	     { RenderAPEItem("Performer",      value, buffer); numItems++; }
 		else if	(key == INFO_CONDUCTOR)	     { RenderAPEItem("Conductor",      value, buffer); numItems++; }
 		else if	(key == INFO_REMIXER)	     { RenderAPEItem("MixArtist",      value, buffer); numItems++; }
@@ -141,11 +142,16 @@ Error BoCA::TaggerAPEv2::RenderBuffer(Buffer<UnsignedByte> &buffer, const Track
 		else if	(key == INFO_PRODUCER)	     { RenderAPEItem("Producer",       value, buffer); numItems++; }
 		else if	(key == INFO_ENGINEER)	     { RenderAPEItem("Engineer",       value, buffer); numItems++; }
 
+		else if	(key == INFO_MOVEMENT)	     { RenderAPEItem("Movement",       value, buffer); numItems++; }
+		else if	(key == INFO_MOVEMENTTOTAL)  { RenderAPEItem("MovementTotal",  value, buffer); numItems++; }
+		else if	(key == INFO_MOVEMENTNAME)   { RenderAPEItem("MovementName",   value, buffer); numItems++; }
+
 		else if	(key == INFO_BPM)	     { RenderAPEItem("BPM",	       value, buffer); numItems++; }
 		else if	(key == INFO_INITIALKEY)     { RenderAPEItem("InitialKey",     value, buffer); numItems++; }
 
 		else if	(key == INFO_COPYRIGHT)	     { RenderAPEItem("Copyright",      value, buffer); numItems++; }
 
+		else if	(key == INFO_MEDIATYPE)	     { RenderAPEItem("Media",	       value, buffer); numItems++; }
 		else if	(key == INFO_CATALOGNUMBER)  { RenderAPEItem("CatalogNumber",  value, buffer); numItems++; }
 		else if	(key == INFO_BARCODE)	     { RenderAPEItem("Barcode",	       value, buffer); numItems++; }
 
@@ -153,7 +159,11 @@ Error BoCA::TaggerAPEv2::RenderBuffer(Buffer<UnsignedByte> &buffer, const Track
 
 		else if	(key == INFO_DISCSUBTITLE)   { RenderAPEItem("DiscSubtitle",   value, buffer); numItems++; }
 
-		else if	(key == INFO_WEB_ARTIST)     { RenderAPEItem("Weblink",	       value, buffer); numItems++; }
+		else if	(key == INFO_WEB_ARTIST)     { RenderAPEItem("Artist URL",     value, buffer); numItems++; }
+		else if	(key == INFO_WEB_PUBLISHER)  { RenderAPEItem("Publisher URL",  value, buffer); numItems++; }
+		else if	(key == INFO_WEB_SOURCE)     { RenderAPEItem("File URL",       value, buffer); numItems++; }
+		else if	(key == INFO_WEB_COPYRIGHT)  { RenderAPEItem("Copyright URL",  value, buffer); numItems++; }
+		else if	(key == INFO_WEB_COMMERCIAL) { RenderAPEItem("Buy URL",	       value, buffer); numItems++; }
 	}
 
 	/* Save Replay Gain info.
@@ -175,10 +185,14 @@ Error BoCA::TaggerAPEv2::RenderBuffer(Buffer<UnsignedByte> &buffer, const Track
 
 	/* Save CD table of contents.
 	 */
-	if (writeMCDI)
-	{
-		if (info.mcdi.GetData().Size() > 0) { RenderAPEBinaryItem("MCDI", info.mcdi.GetData(), buffer); numItems++; }
-	}
+	if (writeMCDI && info.mcdi.IsValid()) { RenderAPEBinaryItem("MCDI", info.mcdi.GetData(), buffer); numItems++; }
+
+	/* Save encoder version.
+	 */
+	Application	*app = Application::Get();
+
+	{ RenderAPEItem("Tool Name",	app->getClientName.Call(),    buffer); numItems++; }
+	{ RenderAPEItem("Tool Version",	app->getClientVersion.Call(), buffer); numItems++; }
 
 	/* Save cover art.
 	 */
@@ -249,7 +263,7 @@ Error BoCA::TaggerAPEv2::RenderBuffer(Buffer<UnsignedByte> &buffer, const Track
 
 			/* Create temporary cuesheet file.
 			 */
-			String	 cueFile  = S::System::System::GetTempDirectory().Append("cuesheet_temp_").Append(String::FromInt(S::System::System::Clock())).Append(".cue");
+			String	 cueFile  = S::System::System::GetTempDirectory().Append("cuesheet_temp_").Append(String::FromInt(track.fileName.ComputeCRC32())).Append(".cue");
 
 			cuesheet->SetTrackList(chapters);
 			cuesheet->WritePlaylist(cueFile);
@@ -409,6 +423,9 @@ Error BoCA::TaggerAPEv2::ParseBuffer(const Buffer<UnsignedByte> &buffer, Track &
 		else if (id == "GROUPING")	 info.SetOtherInfo(INFO_CONTENTGROUP,	value);
 		else if (id == "SUBTITLE")	 info.SetOtherInfo(INFO_SUBTITLE,	value);
 
+		else if (id == "ENSEMBLE" ||
+			 id == "ORCHESTRA")	 info.SetOtherInfo(INFO_BAND,		value);
+
 		else if (id == "PERFORMER")	 info.SetOtherInfo(INFO_PERFORMER,	value);
 		else if (id == "CONDUCTOR")	 info.SetOtherInfo(INFO_CONDUCTOR,	value);
 		else if (id == "MIXARTIST")	 info.SetOtherInfo(INFO_REMIXER,	value);
@@ -417,11 +434,22 @@ Error BoCA::TaggerAPEv2::ParseBuffer(const Buffer<UnsignedByte> &buffer, Track &
 		else if (id == "PRODUCER")	 info.SetOtherInfo(INFO_PRODUCER,	value);
 		else if (id == "ENGINEER")	 info.SetOtherInfo(INFO_ENGINEER,	value);
 
+		else if (id == "MOVEMENT")
+		{
+			info.SetOtherInfo(INFO_MOVEMENT, String::FromInt(value.ToInt()));
+
+			if (value.Contains("/")) info.SetOtherInfo(INFO_MOVEMENTTOTAL, String::FromInt(value.Tail(value.Length() - value.Find("/") - 1).ToInt()));
+		}
+
+		else if (id == "MOVEMENTTOTAL")	 info.SetOtherInfo(INFO_MOVEMENTTOTAL,	value);
+		else if (id == "MOVEMENTNAME")	 info.SetOtherInfo(INFO_MOVEMENTNAME,	value);
+
 		else if (id == "BPM")		 info.SetOtherInfo(INFO_BPM,		value);
 		else if (id == "INITIALKEY")	 info.SetOtherInfo(INFO_INITIALKEY,	value);
 
 		else if (id == "COPYRIGHT")	 info.SetOtherInfo(INFO_COPYRIGHT,	value);
 
+		else if (id == "MEDIA")		 info.SetOtherInfo(INFO_MEDIATYPE,	value);
 		else if (id == "CATALOGNUMBER")  info.SetOtherInfo(INFO_CATALOGNUMBER,	value);
 		else if (id == "BARCODE")	 info.SetOtherInfo(INFO_BARCODE,	value);
 
@@ -429,7 +457,13 @@ Error BoCA::TaggerAPEv2::ParseBuffer(const Buffer<UnsignedByte> &buffer, Track &
 
 		else if (id == "DISCSUBTITLE")	 info.SetOtherInfo(INFO_DISCSUBTITLE,	value);
 
-		else if (id == "WEBLINK")	 info.SetOtherInfo(INFO_WEB_ARTIST,	value);
+		else if (id == "WEBLINK" ||
+			 id == "ARTIST URL")	 info.SetOtherInfo(INFO_WEB_ARTIST,	value);
+
+		else if (id == "PUBLISHER URL")	 info.SetOtherInfo(INFO_WEB_PUBLISHER,	value);
+		else if (id == "FILE URL")	 info.SetOtherInfo(INFO_WEB_SOURCE,	value);
+		else if (id == "COPYRIGHT URL")	 info.SetOtherInfo(INFO_WEB_COPYRIGHT,	value);
+		else if (id == "BUY URL")	 info.SetOtherInfo(INFO_WEB_COMMERCIAL,	value);
 
 		else if (id == "TRACK")
 		{
@@ -452,19 +486,11 @@ Error BoCA::TaggerAPEv2::ParseBuffer(const Buffer<UnsignedByte> &buffer, Track &
 		}
 		else if (id == "MCDI")
 		{
-			/* Check validity of MCDI data.
+			/* Check validity of MCDI data and assign.
 			 */
 			MCDI	 mcdi(item);
-			Bool	 valid = True;
 
-			for (Int i = 1; i < mcdi.GetNumberOfEntries(); i++)
-			{
-				if (mcdi.GetNthEntryOffset(i - 1) >= mcdi.GetNthEntryOffset(i)) valid = False;
-			}
-
-			/* Found a binary MCDI field.
-			 */
-			if (valid) info.mcdi.SetData(item);
+			if (mcdi.IsValid()) info.mcdi.SetData(item);
 		}
 		else if (id.StartsWith("COVER ART"))
 		{
@@ -510,7 +536,7 @@ Error BoCA::TaggerAPEv2::ParseBuffer(const Buffer<UnsignedByte> &buffer, Track &
 				/* Output cuesheet to temporary file.
 				 */
 				String		 cuesheet = value.Replace("\r\n", "\n");
-				String		 cueFile  = S::System::System::GetTempDirectory().Append("cuesheet_temp_").Append(String::FromInt(S::System::System::Clock())).Append(".cue");
+				String		 cueFile  = S::System::System::GetTempDirectory().Append("cuesheet_temp_").Append(String::FromInt(track.fileName.ComputeCRC32())).Append(".cue");
 				OutStream	 out(STREAM_FILE, cueFile, OS_REPLACE);
 
 				/* Write UTF-8 BOM.
@@ -573,6 +599,10 @@ Error BoCA::TaggerAPEv2::ParseBuffer(const Buffer<UnsignedByte> &buffer, Track &
 	 */
 	if (info.artist == NIL) info.artist = info.GetOtherInfo(INFO_ALBUMARTIST);
 
+	/* Remove sub-tracks if main track has a track number.
+	 */
+	if (info.track > 0) track.tracks.RemoveAll();
+
 	track.SetInfo(info);
 
 	return Success();
