@@ -223,6 +223,10 @@ Error BoCA::TaggerWMA::RenderStreamInfo(const String &fileName, const Track &tra
 
 			else if	(key == INFO_COPYRIGHT)	     RenderWMAStringItem(g_wszWMCopyright,		 value, pHeaderInfo);
 
+			else if	(key == INFO_MEDIATYPE)	     RenderWMAStringItem("WM/Media",			 value, pHeaderInfo);
+			else if	(key == INFO_CATALOGNUMBER)  RenderWMAStringItem("WM/CatalogNo",		 value, pHeaderInfo);
+			else if	(key == INFO_BARCODE)	     RenderWMAStringItem("WM/Barcode",			 value, pHeaderInfo);
+
 			else if	(key == INFO_RADIOSTATION)   RenderWMAStringItem(g_wszWMRadioStationName,	 value, pHeaderInfo);
 			else if	(key == INFO_RADIOOWNER)     RenderWMAStringItem(g_wszWMRadioStationOwner,	 value, pHeaderInfo);
 
@@ -251,13 +255,7 @@ Error BoCA::TaggerWMA::RenderStreamInfo(const String &fileName, const Track &tra
 
 		/* Save CD table of contents.
 		 */
-		if (writeMCDI)
-		{
-			if (info.mcdi.GetData().Size() > 0)
-			{
-				RenderWMABinaryItem(g_wszWMMCDI, info.mcdi.GetData(), pHeaderInfo);
-			}
-		}
+		if (writeMCDI && info.mcdi.IsValid()) RenderWMABinaryItem(g_wszWMMCDI, info.mcdi.GetData(), pHeaderInfo);
 
 		/* Save encoder version.
 		 */
@@ -462,6 +460,10 @@ Error BoCA::TaggerWMA::ParseStreamInfo(const String &fileName, Track &track)
 
 			else if (id == g_wszWMCopyright)	       info.SetOtherInfo(INFO_COPYRIGHT,      value);
 
+			else if (id == "WM/Media")		       info.SetOtherInfo(INFO_MEDIATYPE,      value);
+			else if (id == "WM/CatalogNo")		       info.SetOtherInfo(INFO_CATALOGNUMBER,  value);
+			else if (id == "WM/Barcode")		       info.SetOtherInfo(INFO_BARCODE,	      value);
+
 			else if (id == g_wszWMRadioStationName)	       info.SetOtherInfo(INFO_RADIOSTATION,   value);
 			else if (id == g_wszWMRadioStationOwner)       info.SetOtherInfo(INFO_RADIOOWNER,     value);
 
@@ -522,19 +524,11 @@ Error BoCA::TaggerWMA::ParseStreamInfo(const String &fileName, Track &track)
 
 					memcpy(data, pbValue, cbLength);
 
-					/* Check validity of MCDI data.
+					/* Check validity of MCDI data and assign.
 					 */
 					MCDI	 mcdi(data);
-					Bool	 valid = True;
 
-					for (Int i = 1; i < mcdi.GetNumberOfEntries(); i++)
-					{
-						if (mcdi.GetNthEntryOffset(i - 1) >= mcdi.GetNthEntryOffset(i)) valid = False;
-					}
-
-					/* Found a binary MCDI field.
-					 */
-					if (valid) info.mcdi.SetData(data);
+					if (mcdi.IsValid()) info.mcdi.SetData(data);
 				}
 				else
 				{
@@ -748,6 +742,10 @@ Error BoCA::TaggerWMA::UpdateStreamInfo(const String &fileName, const Track &tra
 
 				    nameStr == g_wszWMCopyright			||
 
+				    nameStr == "WM/Media"			||
+				    nameStr == "WM/CatalogNo"			||
+				    nameStr == "WM/Barcode"			||
+
 				    nameStr == g_wszWMRadioStationName		||
 				    nameStr == g_wszWMRadioStationOwner		||
 
